@@ -8,6 +8,7 @@
 #' @param alternative The alternative hypothesis ("two.sided", "greater", or "less")
 #' @param conf_lev Span for the confidence interval
 #' @param data_filter Expression entered in, e.g., Data > View to filter the dataset in Radiant. The expression should be a string (e.g., "price > 10000")
+#' @param envir Environment to extract data from
 #'
 #' @return A list of variables defined in single_mean as an object of class single_mean
 #'
@@ -21,30 +22,38 @@
 single_mean <- function(
   dataset, var, comp_value = 0,
   alternative = "two.sided", conf_lev = .95,
-  data_filter = ""
+  data_filter = "", envir = parent.frame()
 ) {
 
   df_name <- if (is_string(dataset)) dataset else deparse(substitute(dataset))
-  dataset <- get_data(dataset, var, filt = data_filter, na.rm = FALSE)
+  dataset <- get_data(dataset, var, filt = data_filter, na.rm = FALSE, envir = envir)
 
   ## counting missing values
-  miss <- n_missing(dataset)
+  # miss <- n_missing(dataset)
   ## removing any missing values
-  dataset <- na.omit(dataset)
+  # dataset <- na.omit(dataset)
 
   res <- t.test(dataset[[var]], mu = comp_value, alternative = alternative, conf.level = conf_lev) %>%
     tidy()
 
+  ## from http://www.cookbook-r.com/Graphs/Plotting_means_and_error_bars_(ggplot2)/
+  me_calc <- function(se, n, conf.lev = .95)
+    se * qt(conf.lev / 2 + .5, n - 1)
+
   dat_summary <- summarise_all(dataset,
     list(
-      diff = ~ mean(.) - comp_value,
+      diff = ~ mean(., na.rm = TRUE) - comp_value,
+      mean = ~ mean(., na.rm = TRUE),
+      n = length,
+      n_missing = n_missing,
+      sd = ~ sd(., na.rm = TRUE),
       se = se,
-      mean = mean,
-      sd = sd,
-      n_obs = length
+      me = ~ me_calc(se, n, conf_lev)
     )
   )
-  dat_summary$n_missing <- miss
+
+  # removing unneeded arguments
+  rm(envir, me_calc)
 
   as.list(environment()) %>% add_class("single_mean")
 }
@@ -88,8 +97,9 @@ summary.single_mean <- function(object, dec = 3, ...) {
   ci_perc <- ci_label(object$alternative, object$conf_lev)
 
   ## print summary statistics
-  object$dat_summary[ ,-(1:2)] %>%
-    # round(dec) %>%
+  object$dat_summary %>%
+    select(-1) %>%
+    # select_at(c("mean", "n", "n_missing", "sd", "se", "me")) %>%
     as.data.frame(stringsAsFactors = FALSE) %>%
     format_df(dec = dec, mark = ",") %>%
     print(row.names = FALSE)
@@ -169,7 +179,7 @@ plot.single_mean <- function(
       )
   }
   if ("simulate" %in% plots) {
-    var <- x$dataset[[x$var]]
+    var <- na.omit(x$dataset[[x$var]])
     nr <- length(var)
 
     simdat <- replicate(1000, mean(sample(var, nr, replace = TRUE))) %>%
@@ -209,15 +219,12 @@ plot.single_mean <- function(
       labs(title = paste0("Simulated means if null hyp. is true (", x$var, ")"))
   }
 
-  if (custom) {
-    if (length(plot_list) == 1) {
-      return(plot_list[[1]])
+  if (length(plot_list) > 0) {
+    if (custom) {
+      if (length(plot_list) == 1) plot_list[[1]] else plot_list
     } else {
-      return(plot_list)
+      patchwork::wrap_plots(plot_list, ncol = 1) %>%
+        {if (shiny) . else print(.)}
     }
   }
-
-  sshhr(gridExtra::grid.arrange(grobs = plot_list, ncol = 1)) %>% {
-    if (shiny) . else print(.)
-  }
 }
