@@ -6,6 +6,44 @@ pc_dist <- c(
 
 pc_type <- c("Values" = "values", "Probabilities" = "probs")
 
+make_pc_values_input <- function(lb, lb_init = NA, ub, ub_init = 0) {
+  if(!is_empty(r_state[[lb]])) ub_init <- NA
+  if(!is_empty(r_state[[ub]])) lb_init <- NA
+  div(
+    class = "row",
+    div(
+      class = "col-xs-6",
+      numericInput(lb, "Lower bound:", value = state_init(lb, lb_init))
+    ),
+    div(
+      class = "col-xs-6",
+      numericInput(ub, "Upper bound:", value = state_init(ub, ub_init))
+    )
+  )
+}
+
+make_pc_prob_input <- function(lb, lb_init = NA, ub, ub_init = 0.95) {
+  if(!is_empty(r_state[[lb]])) ub_init <- NA
+  if(!is_empty(r_state[[ub]])) lb_init <- NA
+  div(
+    class = "row",
+    div(
+      class = "col-xs-6",
+      numericInput(
+        lb, "Lower bound:", value = state_init(lb, lb_init),
+        min = 0, max = 1, step = .005
+      )
+    ),
+    div(
+      class = "col-xs-6",
+      numericInput( 
+        ub, "Upper bound:", value = state_init(ub, ub_init),
+        min = 0, max = 1, step = .005
+      )
+    )
+  )
+}
+
 output$ui_pc_pois <- renderUI({
   numericInput(
     "pcp_lambda", "Lambda:",
@@ -16,43 +54,9 @@ output$ui_pc_pois <- renderUI({
 
 output$ui_pc_input_pois <- renderUI({
   if (input$pc_type == "values") {
-    div(
-      class = "row",
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pcp_lb", "Lower bound:",
-          value = state_init("pcp_lb", NA)
-        )
-      ),
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pcp_ub", "Upper bound:",
-          value = state_init("pcp_ub", 3)
-        )
-      )
-    )
+    make_pc_values_input("pcp_lb", lb_init = NA, "pcp_ub", ub_init = 3)
   } else {
-    div(
-      class = "row",
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pcp_plb", "Lower bound:",
-          value = state_init("pcp_plb", NA),
-          min = 0, max = 1, step = .005
-        )
-      ),
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pcp_pub", "Upper bound:",
-          value = state_init("pcp_pub", 0.95),
-          min = 0, max = 1, step = .005
-        )
-      )
-    )
+    make_pc_prob_input("pcp_plb", lb_init = NA, "pcp_pub", ub_init = 0.95)
   }
 })
 
@@ -66,43 +70,9 @@ output$ui_pc_expo <- renderUI({
 
 output$ui_pc_input_expo <- renderUI({
   if (input$pc_type == "values") {
-    div(
-      class = "row",
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pce_lb", "Lower bound:",
-          value = state_init("pce_lb", NA)
-        )
-      ),
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pce_ub", "Upper bound:",
-          value = state_init("pce_ub", 2.996)
-        )
-      )
-    )
+    make_pc_values_input("pce_lb", lb_init = NA, "pce_ub", ub_init = 2.996)
   } else {
-    div(
-      class = "row",
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pce_plb", "Lower bound:",
-          value = state_init("pce_plb", NA),
-          step = .005
-        )
-      ),
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pce_pub", "Upper bound:",
-          value = state_init("pce_pub", 0.95),
-          step = .005
-        )
-      )
-    )
+    make_pc_prob_input("pce_plb", lb_init = NA, "pce_pub", ub_init = 0.95)
   }
 })
 
@@ -121,43 +91,9 @@ output$ui_pc_disc <- renderUI({
 
 output$ui_pc_input_disc <- renderUI({
   if (input$pc_type == "values") {
-    div(
-      class = "row",
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pcd_lb", "Lower bound:",
-          value = state_init("pcd_lb", NA)
-        )
-      ),
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pcd_ub", "Upper bound:",
-          value = state_init("pcd_ub", 3)
-        )
-      )
-    )
+    make_pc_values_input("pcd_lb", lb_init = NA, "pcd_ub", ub_init = 3)
   } else {
-    div(
-      class = "row",
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pcd_plb", "Lower bound:",
-          value = state_init("pcd_plb", NA),
-          step = .005
-        )
-      ),
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pcd_pub", "Upper bound:",
-          value = state_init("pcd_pub", 0.95),
-          step = .005
-        )
-      )
-    )
+    make_pc_prob_input("pcd_plb", lb_init = NA, "pcd_pub", ub_init = 0.95)
   }
 })
 
@@ -177,44 +113,10 @@ output$ui_pc_fdist <- renderUI({
 })
 
 output$ui_pc_input_fdist <- renderUI({
-  if (input$pc_type == "values") {
-    div(
-      class = "row",
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pcf_lb", "Lower bound:",
-          value = state_init("pcf_lb", NA)
-        )
-      ),
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pcf_ub", "Upper bound:",
-          value = state_init("pcf_ub", 2.978)
-        )
-      )
-    )
+  if (input$pc_type == "values")  {
+    make_pc_values_input("pcf_lb", lb_init = NA, "pcf_ub", ub_init = 2.978)
   } else {
-    div(
-      class = "row",
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pcf_plb", "Lower bound:",
-          value = state_init("pcf_plb", NA),
-          step = .005
-        )
-      ),
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pcf_pub", "Upper bound:",
-          value = state_init("pcf_pub", 0.95),
-          step = .005
-        )
-      )
-    )
+    make_pc_prob_input("pcf_plb", lb_init = NA, "pcf_pub", ub_init = 0.95)
   }
 })
 
@@ -228,43 +130,9 @@ output$ui_pc_chisq <- renderUI({
 
 output$ui_pc_input_chisq <- renderUI({
   if (input$pc_type == "values") {
-    div(
-      class = "row",
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pcc_lb", "Lower bound:",
-          value = state_init("pcc_lb", NA)
-        )
-      ),
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pcc_ub", "Upper bound:",
-          value = state_init("pcc_ub", 3.841)
-        )
-      )
-    )
+    make_pc_values_input("pcc_lb", lb_init = NA, "pcc_ub", ub_init = 3.841)
   } else {
-    div(
-      class = "row",
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pcc_plb", "Lower bound:",
-          value = state_init("pcc_plb", NA),
-          step = .005
-        )
-      ),
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pcc_pub", "Upper bound:",
-          value = state_init("pcc_pub", 0.95),
-          step = .005
-        )
-      )
-    )
+    make_pc_prob_input("pcc_plb", lb_init = NA, "pcc_pub", ub_init = 0.95)
   }
 })
 
@@ -286,43 +154,9 @@ output$ui_pc_tdist <- renderUI({
 
 output$ui_pc_input_tdist <- renderUI({
   if (input$pc_type == "values") {
-    div(
-      class = "row",
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pct_lb", "Lower bound:",
-          value = state_init("pct_lb", -Inf)
-        )
-      ),
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pct_ub", "Upper bound:",
-          value = state_init("pct_ub", 2.228)
-        )
-      )
-    )
+    make_pc_values_input("pct_lb", lb_init = -Inf, "pct_ub", ub_init = 2.228)
   } else {
-    div(
-      class = "row",
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pct_plb", "Lower bound:",
-          value = state_init("pct_plb", .025),
-          step = .005
-        )
-      ),
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pct_pub", "Upper bound:",
-          value = state_init("pct_pub", 0.975),
-          step = .005
-        )
-      )
-    )
+    make_pc_prob_input("pct_plb", lb_init = 0.025, "pct_pub", ub_init = 0.975)
   }
 })
 
@@ -351,43 +185,9 @@ output$ui_pc_norm <- renderUI({
 
 output$ui_pc_input_norm <- renderUI({
   if (input$pc_type == "values") {
-    div(
-      class = "row",
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pc_lb", "Lower bound:",
-          value = state_init("pc_lb", -Inf)
-        )
-      ),
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pc_ub", "Upper bound:",
-          value = state_init("pc_ub", 0)
-        )
-      )
-    )
+    make_pc_values_input("pc_lb", lb_init = -Inf, "pc_ub", ub_init = 0)
   } else {
-    div(
-      class = "row",
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pc_plb", "Lower bound:",
-          value = state_init("pc_plb", .025),
-          step = .005
-        )
-      ),
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pc_pub", "Upper bound:",
-          value = state_init("pc_pub", 0.975),
-          step = .005
-        )
-      )
-    )
+    make_pc_prob_input("pc_plb", lb_init = 0.025, "pc_pub", ub_init = 0.975)
   }
 })
 
@@ -416,43 +216,9 @@ output$ui_pc_lnorm <- renderUI({
 
 output$ui_pc_input_lnorm <- renderUI({
   if (input$pc_type == "values") {
-    div(
-      class = "row",
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pcln_lb", "Lower bound:",
-          value = state_init("pcln_lb", 0)
-        )
-      ),
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pcln_ub", "Upper bound:",
-          value = state_init("pcln_ub", 1)
-        )
-      )
-    )
+    make_pc_values_input("pcln_lb", lb_init = 0, "pcln_ub", ub_init = 1)
   } else {
-    div(
-      class = "row",
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pcln_plb", "Lower bound:",
-          value = state_init("pcln_plb", .025),
-          step = .005
-        )
-      ),
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pcln_pub", "Upper bound:",
-          value = state_init("pcln_pub", 0.975),
-          step = .005
-        )
-      )
-    )
+    make_pc_prob_input("pcln_plb", lb_init = 0.025, "pcln_pub", ub_init = 0.975)
   }
 })
 
@@ -481,45 +247,9 @@ output$ui_pc_binom <- renderUI({
 
 output$ui_pc_input_binom <- renderUI({
   if (input$pc_type == "values") {
-    div(
-      class = "row",
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pcb_lb", "Lower bound:",
-          value = state_init("pcb_lb", NA),
-          min = 0, step = 1
-        )
-      ),
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pcb_ub", "Upper bound:",
-          value = state_init("pcb_ub", 3),
-          min = 0, step = 1
-        )
-      )
-    )
+    make_pc_values_input("pcb_lb", lb_init = NA, "pcb_ub", ub_init = 3)
   } else {
-    div(
-      class = "row",
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pcb_plb", "Lower bound:",
-          value = state_init("pcb_plb", NA),
-          min = 0, max = 1, step = .005
-        )
-      ),
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pcb_pub", "Upper bound:",
-          value = state_init("pcb_pub", 0.3),
-          min = 0, max = 1, step = .005
-        )
-      )
-    )
+    make_pc_prob_input("pcb_plb", lb_init = NA, "pcb_pub", ub_init = 0.3)
   }
 })
 
@@ -547,43 +277,9 @@ output$ui_pc_unif <- renderUI({
 
 output$ui_pc_input_unif <- renderUI({
   if (input$pc_type == "values") {
-    div(
-      class = "row",
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pcu_lb", "Lower bound:",
-          value = state_init("pcu_lb", NA)
-        )
-      ),
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pcu_ub", "Upper bound:",
-          value = state_init("pcu_ub", 0.3)
-        )
-      )
-    )
+    make_pc_values_input("pcu_lb", lb_init = NA, "pcu_ub", ub_init = 0.3)
   } else {
-    div(
-      class = "row",
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pcu_plb", "Lower bound:",
-          value = state_init("pcu_plb", NA),
-          step = .005
-        )
-      ),
-      div(
-        class = "col-xs-6",
-        numericInput(
-          "pcu_pub", "Upper bound:",
-          value = state_init("pcu_pub", 0.3),
-          step = .005
-        )
-      )
-    )
+    make_pc_prob_input("pcu_plb", lb_init = NA, "pcu_pub", ub_init = 0.3)
   }
 })
 
@@ -868,6 +564,7 @@ pc_available <- reactive({
 })
 
 observeEvent(input$prob_calc_report, {
+  req(input$pc_dist)
   type <- input$pc_type
   inp <- pc_inputs()
   if (!is.null(type) && type == "probs") {
@@ -877,6 +574,12 @@ observeEvent(input$prob_calc_report, {
     inp_out <- list("", "")
     inp[["pub"]] <- inp[["plb"]] <- NA
   }
+
+  if (input$pc_dist == "disc") {
+    inp$v <- radiant.data::make_vec(inp$v)
+    inp$p <- radiant.data::make_vec(inp$p)
+  }
+
   outputs <- c("summary", "plot")
   update_report(
     inp_main = clean_args(inp, pc_args()),
